/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* cpu.h
   all structs, typdefs, externs, defaults, etc are set up in here */

/* Written by Blake Matheny */

#define CONFIG_FILE "/usr/local/etc/cpu.cfg"
#define CPU_VERSION "cpu -- 1.3.13  3/10/02 Blake Matheny <matheny@dbaseiv.net>"
#define MAX_LEN 256 /* Max for DN, passphrase and homedirectory */
#define DEFAULT_LDAPHOST "127.0.0.1"
#define DEFAULT_LDAPPORT 389
#define DEFAULT_MAX_UIDNUMBER 6000
#define DEFAULT_MIN_UIDNUMBER 1000
#define DEFAULT_MAX_GIDNUMBER 6000
#define DEFAULT_MIN_GIDNUMBER 1000
#define DEFAULT_ID_MAX_PASSES 500

/* The passwd structure. We use all chars here because the openldap API
   requires data type of char for what these are being used for */
struct bpasswd
{
  char *pw_name;        /* Username.  */
  char *pw_passwd;      /* Password.  */
  char *pw_uid;			/* User ID.  */
  char *pw_gid;			/* Group ID.  */
  char *pw_gecos;       /* Real name.  */
  char *pw_dir;         /* Home directory.  */
  char *pw_shell;       /* Shell program.  */
};

/* Most configuration options that aren't going into the directory are 
   specified here */
typedef struct confs {
	char *file;		    /* Storage for configuration file */
	int verbose;		/* Level of verbosity		*/
	int operation;		/* Number of operation. 0 for add,
			               1 for delete */
	char *name[2];		/* Name to add/delete, not DN */
	int cipher;         /* Cipher to use */
	char *hash;         /* specify hash at command line */
	char *skeldir;	    /* skeleton directory, where to copy files
			               from */
	char *passfile;		/* passfile to get info from */
	char *shadfile;		/* shadow file to use to get password from */
	char *homedir;		/* root of home directories */
	int makedir;		/* make home directory or not */
	int remdir;		/* remove home directory or not */
	char *def_dir_perm;	/* default directory permissions */
	char *base;		    /* Store different user base if desired */
    	char *id_max_passes;  /* Get/Set Maximum number of directory passes */
	char *logfac;		/* Logging facility. Set if syslog::y */
	int log;		/* 1 if syslog::y */
	int force;		/* 1 if force override */
} GLOBAL;

typedef struct aptions {
	char *ldaphost;
	int ldapport;
	char *binddn;
	char *bindpass;		/* Bind password */
	char dn[MAX_LEN];
	char *user_object_class[1024];
	char *group_object_class[1024];
    char *newuser[2];
    char *newgroup[2];
	int authmethod;
	char *gecos[2];
    char *cn[2];
    char *description[2];
    char *mail[2];
    char *roomnumber[2];
    char *givenname[2];
    char *sn[2];
	char *password[2];	/* User password */
	char *shadowlastchange[2];
	char *shadowmax[2];
	char *shadowwarning[2];
	char *shadowexpire[2];
	char *shadowflag[2];
    char *shadowmin[2];
    char *shadowinactive[2];
    char *homephone[2];
	char *loginshell[2];
	char *gidnumber[2];
	char *uidnumber[2];
	char *homedirectory[2];
	char *memberuid[2];			/* member to add/delete from group */
} LDAPtions;


/* All function that are called and outside of main */
extern void usage(char *progName, FILE *OUTFILE);
extern int do_ldap_init();
extern int do_ldap_user_add();
extern int do_ldap_user_del();
extern int do_ldap_user_mod();
extern int do_ldap_group_add();
extern int do_ldap_group_del();
extern int do_ldap_group_mod();
extern int do_ldap_unbind();
extern int do_ldap_user_cat();
extern int do_ldap_member_mod(int);
extern char *get_next_uidnum();
extern char *get_next_gidnum();
extern char *set_password(char *passphrase);
extern int copy(char *skeldir, char *homedir);
extern int remdir(char *directory);
extern int filestuff(int filefunct);
extern struct bpasswd *bgetpwent (char *filename, char *name);
extern int lutil_b64_ntop(u_char const *, size_t, char *, size_t);
extern char *md5_hash(char *passphrase);
extern char *smd5_hash(char *passphrase, char *salt);
extern char *ssha1_hash(char *passphrase, char *salt);
extern char *sha1_hash(char *passphrase);
extern char *getToken(char** str, const char* delims);
extern int atoo(char *s);
extern char *btolower(char *baz);
extern int bgetopt(int argc, char *const *argv, const char *optstring);
extern time_t time(time_t *);

extern GLOBAL configs;
extern LDAPtions ldopts;
extern CONFIG *confs;
extern struct bpasswd *passent;
