/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* config.c
   This is the guts of libbconf, a simple library in C for parsing config
   files. People don't like installing multiple things for a simple application
   so I started including it. The struct is documented in bconf.h */

/* Written by Blake Matheny */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <bconf.h>
#include <cpu.h>

char *
print_config(confs, name)
CONFIG *confs;
char *name;
{
	CONFIG *bconf;
	for (bconf = confs->first; bconf != NULL; bconf = bconf->next) {
		if (strcmp(bconf->key, name) == 0) {
			return bconf->value;
		}
	}
	return NULL;
}

void
destroy_config(confs)
CONFIG *confs;
{
	confs = confs->first;
	while (confs->current != NULL) {
		confs->previous = confs->current;
		confs->next = confs->current->next;
		free(confs->previous);
	}
}

void
parse_config(confs, name, variable)
CONFIG *confs;
char *name;
char *variable;
{
	char *Nname;
	char *Nvariable;
	int counter = 0;

	confs->current = malloc(sizeof (CONFIG));

	/* Our first time through, so set confs->first equal to the address
	   confs and increment our counter so we know we've already been here
	 */
	if (confs->first == NULL && confs->count == 0) {
		confs->count++;
		confs->first = confs->current;
	}

	if (confs->previous != NULL)
		confs->previous->next = confs->current;

	for (counter = 0; variable[counter] != '\0'; counter++) {
		if (variable[counter] == '\n')
			variable[counter] = '\0';
	}

	Nname = strdup(name);
	Nvariable = strdup(variable);

	confs->current->key = Nname;
	confs->current->value = Nvariable;

	confs->next = NULL;
	confs->previous = confs->current;
}

CONFIG *
init_config()
{
	CONFIG *confs;
	if ((confs = (CONFIG *) calloc(1, sizeof (CONFIG))) == NULL) {
		return (NULL);
	}
	confs->first = NULL;
	confs->current = NULL;
	confs->previous = NULL;
	confs->count = 0;
	return (confs);
}

int
get_config(confs, config, seper)
CONFIG *confs;
char *config;
char *seper;
{
	FILE *pconfig = NULL;
	char config_char[512];
	int char_size = 511;

	if ((pconfig = fopen(config, "r")) == NULL) {
		return 1;
	} else {

		while (fgets(config_char, char_size, pconfig) != NULL) {
			if ((strspn(config_char, "#") > 0) ||
			    !(strncmp(config_char, "\n", 1))) {
				continue;
			} else {
				char *p = config_char;
				char *tokens[1024];
				int num_tokens = 0;

				while (p != NULL && *p != 0) {
					tokens[num_tokens]=getToken(&p, seper);
					num_tokens++;
				}
				if (num_tokens >= 2) {
					if (strlen(seper) > 1)
						parse_config(confs, tokens[0],
							     tokens[2]);
					else
						parse_config(confs, tokens[0],
							     tokens[1]);
				} else {
					return 2;
				}
			}
		}
	}
	fclose(pconfig);
	return 0;
}
