/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* fileaction.c
   All the functions in here are for copying or removing files. Copy is called
   with the skel_dir and the home directory for the user. Scandir is then used
   to get file names which are copied to the home directory. Remove is called 
   with the users home directory */

/* Written by Blake Matheny */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/param.h>		/* definition for MAXPATHLEN */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>
#include <bconf.h>
#include <cpu.h>

int file_select(struct dirent *entry);
int atoo(char *s);

char pathname[MAXPATHLEN];
mode_t filemode;

int
remdir(directory)
char *directory;
{
    DIR *dirp;
    struct dirent *dp;
    struct stat st;
    char *filename = NULL;

    if ( (dirp = opendir(directory)) == NULL ) {
        if ( configs.log )
            syslog(LOG_INFO, "Error doing opendir(%s)\n", directory);
        perror("opendir: ");

        return 1;
    }
    
    while( (dp = readdir(dirp)) != NULL ) {
        if ( (strcmp(dp->d_name, ".") != 0) && (strcmp(dp->d_name,"..") != 0))
        {
            filename = (char *)malloc(sizeof(char *)*
                (strlen(dp->d_name)+strlen(directory)));
            snprintf(filename, (strlen(directory)+strlen(dp->d_name)+2),
                "%s/%s", directory, dp->d_name);

            if ( lstat(filename, &st) == -1)
            {
                if ( configs.log )
                    syslog(LOG_INFO, "Error doing lstat(%s)\n", filename);
                perror("lstat: ");
            }
            

            if ( S_ISDIR(st.st_mode) ) {
                if ( rmdir(filename) == -1 )
                    remdir(filename);
            } else {
                if ( unlink(filename) == -1 )
                {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing unlink(%s)\n", filename);
                    perror("unlink: ");
                }
            }
            free(filename);
        }
    }
    if ( rmdir(directory) == -1 )
    {
        if ( configs.log )
            syslog(LOG_INFO, "Error doing rmdir(%s)\n", directory);
        perror("rmdir: ");
    }
    
    if ( closedir(dirp) == -1 )
    {
        if ( !configs.log )
            fprintf(stdout, "Error doing closedir(), probably nothing to worry about\n");
        else
            syslog(LOG_INFO, "Error doing closedir(), probably nothing to worry about\n");
    }

    return 0;
}

int
copy(directory, newdirectory)
char *directory;
char *newdirectory;
{
    DIR *dirp;
    struct dirent *dp;
    struct stat st;
    char *skelfile = NULL;
    char *homefile = NULL;
    int fd1, fd2;
    void *data;

    if ( (dirp = opendir(directory)) == NULL )
    {
        if ( configs.log )
            syslog(LOG_INFO, "Error doing opendir(%s)\n", directory);
        perror("opendir: ");
        return 1;
    }
    
    while( (dp = readdir(dirp)) != NULL ) {
        if ( (strcmp(dp->d_name, ".") != 0) && (strcmp(dp->d_name,"..") != 0))
        {
            skelfile = (char *)malloc(sizeof(char *)*
                (strlen(dp->d_name)+strlen(directory)+2));
            snprintf(skelfile, (strlen(directory)+strlen(dp->d_name)+2),
                "%s/%s", directory, dp->d_name);

            homefile = (char *)malloc(sizeof(char *)*
                (strlen(dp->d_name)+strlen(newdirectory)+2));
            snprintf(homefile, (strlen(newdirectory)+strlen(dp->d_name)+2),
                "%s/%s", newdirectory, dp->d_name);

            if ( lstat(skelfile, &st) == -1)
            {
                if ( configs.log )
                    syslog(LOG_INFO, "Error doing lstat(%s)\n", skelfile);
                perror("lstat: ");
            }

            if ( S_ISDIR(st.st_mode) ) {
                mkdir(homefile, atoo(configs.def_dir_perm));
                if ( chown(homefile, atoi(ldopts.uidnumber[0]), atoi(ldopts.gidnumber[0])) == -1 )
                {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing chown(%s)\n", homefile);
                    perror("chown: ");
                }
                if ( configs.verbose && configs.log )
                    syslog(LOG_INFO, "mkdir(%s, %s)\n", homefile, configs.def_dir_perm);
                else if ( configs.verbose )
                    fprintf(stdout, "mkdir(%s, %s)\n", homefile, configs.def_dir_perm);

                copy(skelfile, homefile);
            } else {
                if ( (fd1 = open(skelfile, O_RDONLY)) == -1 )
                {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing open(%s)\n", skelfile);
                    perror("open: ");

                    continue;
                }
                data = malloc(st.st_size); 
                
                if ( read(fd1, data, st.st_size) == -1 )
                {
                    perror("read: ");
                    free(data);
                    close(fd1);
                    continue;
                }
                close(fd1);
               
                if ( (fd2 = open(homefile, O_CREAT|O_EXCL|O_WRONLY)) == -1 ) {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing open(%s)\n", homefile);
                    perror("open: ");

                    free(data);
                    continue;
                }

                if ( write(fd2, data, st.st_size) == -1 )
                    perror("write: ");
                close(fd2);
                free(data);

                if ( configs.verbose && configs.log )
                    syslog(LOG_INFO, "chown(%s, %s, %s)\n", homefile, ldopts.uidnumber[0], ldopts.gidnumber[0]);
                else
                    fprintf(stdout, "chown(%s, %s, %s)\n", homefile,
                        ldopts.uidnumber[0], ldopts.gidnumber[0]);

                if ( chown(homefile, atoi(ldopts.uidnumber[0]), atoi(ldopts.gidnumber[0])) == -1)
                {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing chown(%s)\n", homefile);
                    perror("chown: ");
                }

                if ( chmod(homefile, st.st_mode) == -1)
                {
                    if ( configs.log )
                        syslog(LOG_INFO, "Error doing chmod(%s)\n", homefile);
                    perror("chmod: ");
                }

            }
            free(skelfile);
            free(homefile);
        }
    }
    closedir(dirp);

    return 0;
}

int
atoo(char *s)
{
	int n = 0;

	while ('0' <= *s && *s < '8') {
		n <<= 3;
		n += *s++ - '0';
	}
	return n;
}
