/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* ldap.c
   All LDAP functions are taken care of. LDAP_INIT is called to initialize
   everything and do some sanity checking. LDAP_ADD and LDAP_DELETE functions
   add and delete entries. LDAP_MOD has not yet been implemented. */

/* Written by Blake Matheny */

#include <stdio.h>
#include <stdlib.h>
#include <bconf.h>
#include <lber.h>
#include <ldap.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <syslog.h>
#include <cpu.h>

LDAP *ld;
static void do_other_work();
unsigned long global_counter;

int
do_ldap_init()
{
	char *portnum;
	int sizeChar = 0;

	if ((ldopts.ldaphost = print_config(confs, "ldap_host")) == NULL)
		ldopts.ldaphost = DEFAULT_LDAPHOST;

	if ((portnum = print_config(confs, "ldap_port")) == NULL)
		ldopts.ldapport = DEFAULT_LDAPPORT;
	else
		ldopts.ldapport = atoi(portnum);

	if (!ldopts.binddn) {
		if ((ldopts.binddn = print_config(confs, "bind_dn")) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"bind dn not found in config file and not "
					"specified at command line, now exiting.\n");
			else
				syslog(LOG_INFO,
				       "bind dn not found in config file and not "
				       "specified at command line, now exiting.\n");
			exit(1);
		}
	}

	if (!ldopts.bindpass) {
		if ((ldopts.bindpass =
		     print_config(confs, "bind_pass")) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"bind pass not found in config file and not "
					"specified at command line, now exiting.\n");
			else
				syslog(LOG_INFO,
				       "bind pass not found in config file and not "
				       "specified at command line, now exiting.\n");

			exit(1);
		}
	}

	if (!ldopts.gecos[0] && configs.operation == 0) {
		if ((ldopts.gecos[0] =
		     (char *) malloc(sizeof (char) * MAX_LEN)) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"Unable to allocate memory for gecos. Exiting.\n");
			else
				syslog(LOG_INFO,
				       "Unable to allocate memory for gecos. Exiting.\n");
			exit(1);
		}

		if ((configs.passfile =
		     print_config(confs, "password_file")) == NULL) {
			if ((ldopts.gecos[0] =
			     print_config(confs, "def_gecos")) == NULL)
				ldopts.gecos[0] = "LDAP User1";
		} else
		    if ((passent =
			 bgetpwent(configs.passfile,
				   configs.name[0])) == NULL) {
			if ((ldopts.gecos[0] =
			     print_config(confs, "def_gecos")) == NULL)
				ldopts.gecos[0] = "LDAP User2";
		} else {
			if (strcmp(passent->pw_gecos, "") == 0)
				ldopts.gecos[0] = "LDAP User3";
			else
				ldopts.gecos[0] = strdup(passent->pw_gecos);
		}
	}
	ldopts.gecos[1] = NULL;

	if (ldopts.givenname[0] && ldopts.sn[0] && configs.operation == 0) {
		sizeChar = (strlen(ldopts.givenname[0]) + strlen(ldopts.sn[0]));
		if ((ldopts.cn[0] =
		     (char *) malloc(sizeChar * sizeof (char *))) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"Unable to allocate memory for CN. Exiting.\n");
			else
				syslog(LOG_INFO,
				       "Unable to allocate memory for CN. Exiting.\n");

			exit(1);
		}
		snprintf(ldopts.cn[0], sizeChar + 2, "%s %s",
			 ldopts.givenname[0], ldopts.sn[0]);
	} else if (configs.operation == 0)
		ldopts.cn[0] = configs.name[0];
	ldopts.cn[1] = NULL;

	if (!ldopts.mail[0] && configs.operation == 0) {
		if ((ldopts.mail[0] =
		     (char *) malloc(strlen(configs.name[0]) *
				     sizeof (char *))) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"Unable to allocate memory for Mail. Exiting.\n");
			else
				syslog(LOG_INFO,
				       "Unable to allocate memory for Mail. Exiting.\n");
			exit(1);
		}
		snprintf(ldopts.mail[0],
			 strlen(configs.name[0]) + 2, "%s@", configs.name[0]);
	}
	ldopts.mail[1] = NULL;

	if (!ldopts.givenname[0] && configs.operation == 0)
		ldopts.givenname[0] = "Empty";
	ldopts.givenname[1] = NULL;

	if (!ldopts.sn[0] && configs.operation == 0)
		ldopts.sn[0] = "Empty";
	ldopts.sn[1] = NULL;

	ldopts.roomnumber[0] = "Empty";
	ldopts.roomnumber[1] = NULL;

	ldopts.description[0] = "User";
	ldopts.description[1] = NULL;

	if (!ldopts.gidnumber[0] && (configs.operation == 0)) {
		if ((configs.passfile =
		     print_config(confs, "password_file")) == NULL) {
			if ((ldopts.gidnumber[0] =
			     print_config(confs, "default_gid")) == NULL) {
				if (!configs.log)
					fprintf(stdout,
						"gid not specified at command line and "
						"password_file not specified in config file, "
						"now exiting.\n");
				else
					syslog(LOG_INFO,
					       "Gid not specified at command line and "
					       "password_file not specified in config file, now "
					       "exiting.\n");
				exit(1);
			}
		}
		if ((passent =
		     bgetpwent(configs.passfile, configs.name[0])) == NULL) {
			if ((ldopts.gidnumber[0] =
			     print_config(confs, "default_gid")) == NULL) {
				if (!configs.log)
					fprintf(stdout,
						"gid not specified at command line and %s not "
						"found in %s, now exiting.\n",
						configs.name[0],
						configs.passfile);
				else
					syslog(LOG_INFO,
					       "gid not specified at command line and %s not "
					       "found in %s, now exiting.\n",
					       configs.name[0],
					       configs.passfile);
				exit(1);
			}
		} else
			ldopts.gidnumber[0] = strdup(passent->pw_gid);
	}
	ldopts.gidnumber[1] = NULL;

	ldopts.homedirectory[0] = malloc(sizeof (LDAPtions));
	if (!configs.homedir && (configs.operation == 0 || configs.remdir)) {
		if ((configs.homedir =
		     print_config(confs, "home_directory")) == NULL) {
			if (!configs.log)
				fprintf(stdout,
					"home directory not specified at command line "
					"and home_directory not specified in config "
					"file, now exiting.\n");
			else
				syslog(LOG_INFO,
				       "home directory not specified at command line "
				       "and home_directory not specified in config "
				       "file, now exiting.\n");
			exit(1);
		} else
			snprintf(ldopts.homedirectory[0], MAX_LEN + 1, "%s/%s",
				 configs.homedir, configs.name[0]);
	} else if (configs.operation != 6)
		snprintf(ldopts.homedirectory[0], MAX_LEN, "%s",
			 configs.homedir);
	ldopts.homedirectory[1] = NULL;

	if ((ldopts.shadowlastchange[0] =
	     print_config(confs, "shadowlastchange")) == NULL)
		ldopts.shadowlastchange[0] = "11192";
	ldopts.shadowlastchange[1] = NULL;

	if ((ldopts.shadowmax[0] = print_config(confs, "shadowmax")) == NULL)
		ldopts.shadowmax[0] = "99999";
	ldopts.shadowmax[1] = NULL;

	if ((ldopts.shadowwarning[0] =
	     print_config(confs, "shadowwarning")) == NULL)
		ldopts.shadowwarning[0] = "7";
	ldopts.shadowwarning[1] = NULL;

	if ((ldopts.shadowexpire[0] =
	     print_config(confs, "shadowexpire")) == NULL)
		ldopts.shadowexpire[0] = "-1";
	ldopts.shadowexpire[1] = NULL;

	if ((ldopts.shadowmin[0] = print_config(confs, "shadowmin")) == NULL)
		ldopts.shadowmin[0] = "-1";
	ldopts.shadowmin[1] = NULL;

	if ((ldopts.shadowinactive[0] =
	     print_config(confs, "shadowinactive")) == NULL)
		ldopts.shadowinactive[0] = "-1";
	ldopts.shadowinactive[1] = NULL;

	if ((ldopts.homephone[0] = print_config(confs, "homephone")) == NULL)
		ldopts.homephone[0] = "212-555-5555";
	ldopts.homephone[1] = NULL;

	if ((ldopts.shadowflag[0] = print_config(confs, "shadowflag")) == NULL)
		ldopts.shadowflag[0] = "134538308";
	ldopts.shadowflag[1] = NULL;

	if (!ldopts.loginshell[0] && configs.operation == 0) {
		if ((ldopts.loginshell[0] =
		     print_config(confs, "default_shell")) == NULL)
			ldopts.loginshell[0] = "/bin/sh";
	}
	ldopts.loginshell[1] = NULL;

	ldopts.authmethod = LDAP_AUTH_SIMPLE;

	if (!ldopts.newuser[0] &&
	    (configs.operation == 0 || configs.operation == 1
	     || configs.operation == 2)) {
		if (configs.base)
			snprintf(ldopts.dn, MAX_LEN - 1, "uid=%s,%s",
				 configs.name[0], configs.base);
		else
			snprintf(ldopts.dn, MAX_LEN - 1, "uid=%s,%s",
				 configs.name[0], print_config(confs,
							       "user_base"));
	}
	if (!ldopts.newgroup[0]
	    && (configs.operation == 3 || configs.operation == 4
		|| configs.operation == 5)) {
		if (configs.base)
			snprintf(ldopts.dn, MAX_LEN - 1, "cn=%s,%s",
				 configs.name[0], configs.base);
		else
			snprintf(ldopts.dn, MAX_LEN - 1, "cn=%s,%s",
				 configs.name[0], print_config(confs,
							       "group_base"));
	}

	if ((configs.id_max_passes =
	     print_config(confs, "id_max_passes")) == NULL)
		configs.id_max_passes = NULL;

	if ((ld = ldap_init(ldopts.ldaphost, ldopts.ldapport)) == NULL) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_init(%s, %d)\n",
			       ldopts.ldaphost, ldopts.ldapport);
		perror("ldap_init");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_init( %s, %d )\n",
		       ldopts.ldaphost, ldopts.ldapport);
	else if (configs.verbose)
		fprintf(stdout, "ldap_init( %s, %d )\n",
			ldopts.ldaphost, ldopts.ldapport);

	if (ldap_bind_s(ld, ldopts.binddn, ldopts.bindpass, ldopts.authmethod)
	    != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO,
			       "Error in ldap_bind(ld, %s, PASS, %x)\n",
			       ldopts.binddn, ldopts.authmethod);
		ldap_perror(ld, "ldap_bind");
		return -2;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_bind(ld, %s, PASS, %x)\n",
		       ldopts.binddn, ldopts.authmethod);
	else if (configs.verbose)
		fprintf(stdout, "ldap_bind(ld, %s, PASS, %x)\n",
			ldopts.binddn, ldopts.authmethod);

	if (!ldopts.uidnumber[0] && (configs.operation == 0)) {
		if ((configs.passfile =
		     print_config(confs, "password_file")) != NULL) {
			if ((passent =
			     bgetpwent(configs.passfile,
				       configs.name[0])) != NULL)
				ldopts.uidnumber[0] = strdup(passent->pw_uid);
		}

		if (!ldopts.uidnumber[0]) {
			if ((ldopts.uidnumber[0] = get_next_uidnum()) == NULL) {
				if (!configs.log)
					fprintf(stdout,
						"A new uidnumber was not specified at the command line, could not be\n"
						"found in password_file (or password_file was not specified) and a uid\n"
						"number could not be generated for you. Now exiting.\n");
				else
					syslog(LOG_INFO,
					       "A new uidnumber was not specified at the command line, could not be\n"
					       "found in password_file (or password_file was not specified) and a uid\n"
					       "number could not be generated for you. Now exiting.\n");
				return 1;
			}
		}
	}
	ldopts.uidnumber[1] = NULL;

	if (!ldopts.gidnumber[0] && configs.operation == 3
	    && !ldopts.memberuid[0]) {
		if ((ldopts.gidnumber[0] = get_next_gidnum()) == NULL) {
			if (!configs.log)
				fprintf(stderr,
					"A gidnumber was not specified at the command line and a new gid number could "
					"not be generated for you. Now exiting.\n");
			else
				syslog(LOG_INFO,
				       "A gidnumber was not specified at the command line and a new gid number could "
				       "not be generated for you. Now exiting.\n");
			return 1;
		}
	}

	return 0;
}

int
do_ldap_group_mod()
{
	LDAPMod **mods;
	int num_mods = 0;
	int rc, i;
	struct foo {
		int objectclass;	/* always true */
		int gidnum;	/* if gidnumber foo.gidnumber is 1 */
		char oldgidname[MAX_LEN];	/* place to store old dn */
		char newgroup[MAX_LEN];	/* temporary place for rdn */
	};
	struct foo bar;

	if (ldopts.newgroup[0]) {
		if (configs.base) {
			snprintf(ldopts.dn, MAX_LEN, "cn=%s,%s",
				 ldopts.newgroup[0], configs.base);
			snprintf(bar.oldgidname, MAX_LEN, "cn=%s,%s",
				 configs.name[0], configs.base);
		} else {
			snprintf(ldopts.dn, MAX_LEN, "cn=%s,%s",
				 ldopts.newgroup[0], print_config(confs,
								  "group_base"));
			snprintf(bar.oldgidname, MAX_LEN, "cn=%s,%s",
				 configs.name[0], print_config(confs,
							       "group_base"));
		}
		snprintf(bar.newgroup, MAX_LEN, "cn=%s", ldopts.newgroup[0]);

		if (configs.verbose && configs.log)
			syslog(LOG_INFO,
			       "ldap_modrdn2_s( ld , %s , %s , 1)\n",
			       bar.oldgidname, ldopts.newgroup[0]);
		else if (configs.verbose)
			fprintf(stdout,
				"ldap_modrdn2_s( ld , %s , %s , 1)\n",
				bar.oldgidname, ldopts.newgroup[0]);

		if ((rc =
		     ldap_modrdn2_s(ld, bar.oldgidname, bar.newgroup,
				    1)) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO,
				       "Error doing ldap_modrdn2_s( ld , %s , %s, 1)\n",
				       bar.oldgidname, ldopts.newgroup[0]);
			ldap_perror(ld, "ldap_modrdn2_s");
			return -1;
		}
	} else {
		if (configs.base)
			snprintf(ldopts.dn, MAX_LEN + 1, "cn=%s,%s",
				 configs.name[0], configs.base);
		else
			snprintf(ldopts.dn, MAX_LEN + 1, "cn=%s,%s",
				 configs.name[0], print_config(confs,
							       "group_base"));
	}

	if (ldopts.gidnumber[0]) {
		bar.gidnum = 1;
		num_mods++;
		ldopts.gidnumber[1] = NULL;
	}

	if (num_mods != 0) {
		mods = (LDAPMod **) malloc((num_mods + 1) * sizeof (LDAPMod *));
		if (mods == NULL) {
			if (!configs.log)
				fprintf(stderr,
					"Cannot allocate memory for mods array\n");
			else
				syslog(LOG_INFO,
				       "Cannot allocate memory for mods array\n");

			exit(1);
		}

		for (i = 0; i < num_mods; i++) {
			if ((mods[i] =
			     (LDAPMod *) malloc(sizeof (LDAPMod))) == NULL) {
				if (!configs.log)
					fprintf(stderr,
						"Cannot allocate memory for mods element(s)\n");
				else
					syslog(LOG_INFO,
					       "Cannot allocate memory for mods element(s)\n");
				exit(1);
			}
		}

		for (i = 0; i < num_mods; i++) {
			if (bar.gidnum == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "gidNumber";
				mods[i]->mod_values = ldopts.gidnumber;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing gidnumber of %s to %s\n",
					       ldopts.dn, ldopts.gidnumber[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing gidnumber of %s to %s\n",
						ldopts.dn, ldopts.gidnumber[0]);
				bar.gidnum = 0;
				continue;
			}
		}
		mods[num_mods] = NULL;

		if (configs.verbose && configs.log)
			syslog(LOG_INFO, "ldap_modify_s(%s)\n", ldopts.dn);
		else if (configs.verbose)
			fprintf(stdout, "ldap_modify_s( %s )\n", ldopts.dn);

		if ((rc = ldap_modify_s(ld, ldopts.dn, mods)) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO,
				       "Error doing ldap_modify_s(ld, %s, mods)\n",
				       ldopts.dn);
			ldap_perror(ld, "ldap_modify_s");
			return -1;
		}
	} else if (!ldopts.newgroup[0]) {
		if (!configs.log)
			fprintf(stdout,
				"groupmod requested, but no modifications done.\n");
		else
			syslog(LOG_INFO,
			       "groupmod requested, but no modifications done.\n");
	}

	return 0;
}

int
do_ldap_user_mod()
{
	LDAPMod **mods;
	char passphrase[MAX_LEN];
	int num_mods = 0;
	int rc, i;
	struct foo {
		int password;	/* if password foo.password is 1 */
		int objectclass;	/* always true */
		int uidnum;	/* if uidnumber != "" foo.uidnum is 1 */
		int gidnum;	/* if gidnumber != "" foo.gidnum is 1 */
		int gecos;	/* if gecos foo.gecos is 1 */
		int loginshell;	/* if loginshell foo.loginshell is 1 */
		int mail;	/* if mail foo.mail is 1 */
		int givenname;	/* if givenname foo.givenname is 1 */
		int sn;		/* if sn foo.sn is 1 */
		char olduidname[MAX_LEN];	/* place to store old dn */
		char newuid[MAX_LEN];	/* temporary place for rdn */
	};
	struct foo bar;

	if (ldopts.newuser[0]) {
		if (configs.base) {
			snprintf(ldopts.dn, MAX_LEN, "uid=%s,%s",
				 ldopts.newuser[0], configs.base);
			snprintf(bar.olduidname, MAX_LEN, "uid=%s,%s",
				 configs.name[0], configs.base);
		} else {
			snprintf(ldopts.dn, MAX_LEN, "uid=%s,%s",
				 ldopts.newuser[0], print_config(confs,
								 "user_base"));
			snprintf(bar.olduidname, MAX_LEN, "uid=%s,%s",
				 configs.name[0], print_config(confs,
							       "user_base"));
		}
		snprintf(bar.newuid, MAX_LEN, "uid=%s", ldopts.newuser[0]);
		if (configs.verbose && configs.log)
			syslog(LOG_INFO, "ldap_modrdn2_s( ld, %s , %s , 1 )\n",
			       bar.olduidname, ldopts.newuser[0]);
		else if (configs.verbose)
			fprintf(stdout, "ldap_modrdn2_s( ld , %s , %s , 1 )\n",
				bar.olduidname, ldopts.newuser[0]);

		if ((rc =
		     ldap_modrdn2_s(ld, bar.olduidname, bar.newuid,
				    1)) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO,
				       "Error doing ldap_modrdn2_s( ld , %s , %s , 1 )\n",
				       bar.olduidname, bar.newuid);
			ldap_perror(ld, " ldap_modrdn2_s ");
			return -1;
		}
	} else {
		if (configs.base)
			snprintf(ldopts.dn, MAX_LEN, "uid=%s,%s",
				 configs.name[0], configs.base);
		else
			snprintf(ldopts.dn, MAX_LEN, "uid=%s,%s",
				 configs.name[0], print_config(confs,
							       "user_base"));
	}

	if (configs.cipher != 0) {
		if (configs.cipher == 1)
			snprintf(passphrase, MAX_LEN, "{crypt}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 2)
			snprintf(passphrase, MAX_LEN, "{md5}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 3)
			snprintf(passphrase, MAX_LEN, "{sha}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 4)
			snprintf(passphrase, MAX_LEN, "{smd5}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 5)
			snprintf(passphrase, MAX_LEN, "{ssha}%s",
				 ldopts.password[0]);
		ldopts.password[0] = passphrase;
		ldopts.password[1] = NULL;
		bar.password = 1;
		num_mods++;
	}

	if (strncmp(ldopts.uidnumber[0], "0", 1) == 0) {
		if (!configs.log)
			fprintf(stdout,
				"User requested uid of 0. I'm sorry Dave, I can't let you do this.\n");
		else
			syslog(LOG_INFO,
			       "User requested uid of 0. I'm sorry Dave, I can't let you do this.\n");
		return 0;
	}

	if (strcmp(ldopts.uidnumber[0], "") != 0) {
		bar.uidnum = 1;
		num_mods++;
		ldopts.uidnumber[1] = NULL;
	}

	if (strcmp(ldopts.gidnumber[0], "") != 0) {
		bar.gidnum = 1;
		num_mods++;
		ldopts.gidnumber[1] = NULL;
	}

	if (ldopts.gecos[0]) {
		bar.gecos = 1;
		num_mods++;
		ldopts.gecos[1] = NULL;
	}

	if (ldopts.loginshell[0]) {
		bar.loginshell = 1;
		num_mods++;
		ldopts.loginshell[1] = NULL;
	}

	if (ldopts.mail[0]) {
		bar.mail = 1;
		num_mods++;
		ldopts.mail[1] = NULL;
	}

	if (ldopts.givenname[0]) {
		bar.givenname = 1;
		num_mods++;
		ldopts.givenname[1] = NULL;
	}

	if (ldopts.sn[0]) {
		bar.sn = 1;
		num_mods++;
		ldopts.sn[1] = NULL;
	}

	if (num_mods != 0) {
		mods = (LDAPMod **) malloc((num_mods + 1) * sizeof (LDAPMod *));
		if (mods == NULL) {
			if (!configs.log)
				fprintf(stderr,
					"Cannot allocate memory for mods array\n");
			else
				syslog(LOG_INFO,
				       "Cannot allocate memory for mods array\n");
			exit(1);
		}

		for (i = 0; i < num_mods; i++) {
			if ((mods[i] =
			     (LDAPMod *) malloc(sizeof (LDAPMod))) == NULL) {
				if (!configs.log)
					fprintf(stderr,
						"Cannot allocate memory for mods element(s)\n");
				else
					syslog(LOG_INFO,
					       "Cannot allocate memory for mods array\n");
				exit(1);
			}
		}

		for (i = 0; i < num_mods; i++) {
			if (bar.password == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "userPassword";
				mods[i]->mod_values = ldopts.password;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing password of %s to %s\n",
					       ldopts.dn, ldopts.password[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing password of %s to %s\n",
						ldopts.dn, ldopts.password[0]);
				bar.password = 0;
				continue;
			}
			if (bar.uidnum == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "uidNumber";
				mods[i]->mod_values = ldopts.uidnumber;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing uidNumber of %s to %s\n",
					       ldopts.dn, ldopts.uidnumber[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing uidNumber of %s to %s\n",
						ldopts.dn, ldopts.uidnumber[0]);
				bar.uidnum = 0;
				continue;
			}
			if (bar.gidnum == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "gidNumber";
				mods[i]->mod_values = ldopts.gidnumber;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing gidnumber of %s to %s\n",
					       ldopts.dn, ldopts.gidnumber[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing gidnumber of %s to %s\n",
						ldopts.dn, ldopts.gidnumber[0]);
				bar.gidnum = 0;
				continue;
			}
			if (bar.gecos == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "gecos";
				mods[i]->mod_values = ldopts.gecos;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing gecos entry of %s to %s\n",
					       ldopts.dn, ldopts.gecos[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing gecos entry of %s to %s\n",
						ldopts.dn, ldopts.gecos[0]);
				bar.gecos = 0;
				continue;
			}
			if (bar.loginshell == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "loginShell";
				mods[i]->mod_values = ldopts.loginshell;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing loginshell of %s to %s\n",
					       ldopts.dn, ldopts.loginshell[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing loginshell of %s to %s\n",
						ldopts.dn,
						ldopts.loginshell[0]);
				bar.loginshell = 0;
				continue;
			}
			if (bar.mail == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "mail";
				mods[i]->mod_values = ldopts.mail;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing mail address of %s to %s\n",
					       ldopts.dn, ldopts.mail[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing mail address of %s to %s\n",
						ldopts.dn, ldopts.mail[0]);
				bar.mail = 0;
				continue;
			}
			if (bar.givenname == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "givenname";
				mods[i]->mod_values = ldopts.givenname;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing givenname of %s to %s\n",
					       ldopts.dn, ldopts.givenname[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing givenname of %s to %s\n",
						ldopts.dn, ldopts.givenname[0]);
				bar.givenname = 0;
				continue;
			}
			if (bar.sn == 1) {
				mods[i]->mod_op = LDAP_MOD_REPLACE;
				mods[i]->mod_type = "sn";
				mods[i]->mod_values = ldopts.sn;
				if (configs.verbose && configs.log)
					syslog(LOG_INFO,
					       "Changing sn of %s to %s\n",
					       ldopts.dn, ldopts.sn[0]);
				else if (configs.verbose)
					fprintf(stdout,
						"Changing sn of %s to %s\n",
						ldopts.dn, ldopts.sn[0]);
				bar.sn = 0;
				continue;
			}
		}
		mods[num_mods] = NULL;

		if (configs.verbose && configs.log)
			syslog(LOG_INFO, "ldap_modify_s( %s )\n", ldopts.dn);
		else if (configs.verbose)
			fprintf(stdout, "ldap_modify_s( %s )\n", ldopts.dn);

		if ((rc = ldap_modify_s(ld, ldopts.dn, mods)) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO, "Error doing ldap_modify_s\n");
			ldap_perror(ld, " ldap_modify_s");
			return -1;
		}

	} else if (!ldopts.newuser[0]) {
		if (!configs.log)
			fprintf(stdout,
				"usermod requested, but no modifications done.\n");
		else
			syslog(LOG_INFO,
			       "usermod requested, but no modifications done.\n");
	}

	return 0;
}

int
do_ldap_member_mod(operation)
int operation;
{
	LDAPMod **mods;
	int rc;
	int num_mods = 1;

	mods = (LDAPMod **) malloc((num_mods + 1) * sizeof (LDAPMod *));

	if (mods == NULL) {
		if (!configs.log)
			fprintf(stderr,
				"Cannot allocate memory for mods array\n");
		else
			syslog(LOG_INFO,
			       "Cannot allocate memory for mods array\n");
		return 1;
	}

	if ((mods[0] = (LDAPMod *) malloc(sizeof (LDAPMod))) == NULL) {
		if (!configs.log)
			fprintf(stderr,
				"Cannot allocate memory for mods elements\n");
		else
			syslog(LOG_INFO,
			       "Cannot allocate memory for mods elements\n");
		return 1;
	}

	mods[0]->mod_op = (operation) ? LDAP_MOD_ADD : LDAP_MOD_DELETE;
	mods[0]->mod_type = "memberUid";
	mods[0]->mod_values = ldopts.memberuid;

	mods[num_mods] = NULL;

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_modify_s( %s )\n", ldopts.dn);
	else if (configs.verbose)
		fprintf(stdout, "ldap_modify_s( %s )\n", ldopts.dn);

	if ((rc = ldap_modify_s(ld, ldopts.dn, mods)) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_modify_s\n");
		ldap_perror(ld, " ldap_modify_s");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "group=%s successfully modified\n",
		       configs.name[0]);
	else if (configs.verbose)
		fprintf(stdout, "\tgroup=%s successfully modified\n",
			configs.name[0]);

	return 0;
}

int
do_ldap_group_add()
{
	LDAPMod **mods;
	char *object_vals = NULL;
	int num_tokens, rc, i;
	int num_mods = 3;
	num_tokens = 0;

	if ((object_vals = print_config(confs, "group_object_class")) == NULL) {
		if (!configs.log)
			fprintf(stdout,
				"group_object_class must be specified in config file\n");
		else
			syslog(LOG_INFO,
			       "group_object_class must be specified in config file\n");
		return 1;
	}

	while (object_vals != NULL && *object_vals) {
		ldopts.group_object_class[num_tokens + 1] = NULL;
		ldopts.group_object_class[num_tokens] =
		    getToken(&object_vals, ",");
		num_tokens++;
	}

	mods = (LDAPMod **) malloc((num_mods + 1) * sizeof (LDAPMod *));

	if (mods == NULL) {
		if (!configs.log)
			fprintf(stderr,
				"Cannot allocate memory for mods array\n");
		else
			syslog(LOG_INFO,
			       "Cannot allocate memory for mods array\n");
		return 1;
	}

	for (i = 0; i < num_mods; i++) {
		if ((mods[i] = (LDAPMod *) malloc(sizeof (LDAPMod))) == NULL) {
			if (!configs.log)
				fprintf(stderr,
					"Cannot allocate memory for mods elements\n");
			else
				syslog(LOG_INFO,
				       "Cannot allocate memory for mods elements\n");
			return 1;
		}
	}

	mods[0]->mod_op = LDAP_MOD_ADD;
	mods[0]->mod_type = "objectclass";
	mods[0]->mod_values = ldopts.group_object_class;

	mods[1]->mod_op = LDAP_MOD_ADD;
	mods[1]->mod_type = "cn";
	mods[1]->mod_values = configs.name;

	mods[2]->mod_op = LDAP_MOD_ADD;
	mods[2]->mod_type = "gidnumber";
	mods[2]->mod_values = ldopts.gidnumber;

	mods[num_mods] = NULL;

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_add_s( %s )\n", ldopts.dn);
	else if (configs.verbose)
		fprintf(stdout, "ldap_add_s( %s )\n", ldopts.dn);

	if ((rc = ldap_add_s(ld, ldopts.dn, mods)) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_add_s\n");
		ldap_perror(ld, " ldap_add_s");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "group=%s successfully added\n",
		       configs.name[0]);
	else if (configs.verbose)
		fprintf(stdout, "\tgroup=%s successfully added\n",
			configs.name[0]);

	return 0;
}

int
do_ldap_user_add()
{
	LDAPMod **mods;
	char *object_vals = NULL;
	char passphrase[MAX_LEN];
	int num_tokens, rc, i;
	int num_mods = 22;
	num_tokens = 0;

	if (configs.cipher != 0) {
		if (configs.cipher == 1)
			snprintf(passphrase, MAX_LEN, "{crypt}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 2)
			snprintf(passphrase, MAX_LEN, "{md5}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 3)
			snprintf(passphrase, MAX_LEN, "{sha}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 4)
			snprintf(passphrase, MAX_LEN, "{smd5}%s",
				 ldopts.password[0]);
		else if (configs.cipher == 5)
			snprintf(passphrase, MAX_LEN, "{ssha}%s",
				 ldopts.password[0]);

		ldopts.password[0] = passphrase;
	}

	if ((object_vals = print_config(confs, "user_object_class")) == NULL) {
		if (!configs.log)
			fprintf(stdout,
				"user_object_class must be specified in config file\n");
		else
			syslog(LOG_INFO,
			       "user_object_class must be specified in config file\n");
		return 1;
	}

	while (object_vals != NULL && *object_vals) {
		ldopts.user_object_class[num_tokens + 1] = NULL;
		ldopts.user_object_class[num_tokens] =
		    getToken(&object_vals, ",");
		num_tokens++;
	}

	mods = (LDAPMod **) malloc((num_mods + 1) * sizeof (LDAPMod *));

	if (mods == NULL) {
		if (!configs.log)
			fprintf(stderr,
				"Cannot allocate memory for mods array\n");
		else
			syslog(LOG_INFO,
			       "Cannot allocate memory for mods array\n");
		return 1;
	}

	for (i = 0; i < num_mods; i++) {
		if ((mods[i] = (LDAPMod *) malloc(sizeof (LDAPMod))) == NULL) {
			if (!configs.log)
				fprintf(stderr,
					"Cannot allocate memory for mods elements\n");
			else
				syslog(LOG_INFO,
				       "Cannot allocate memory for mods elements\n");
			return 1;
		}
	}

	mods[0]->mod_op = LDAP_MOD_ADD;
	mods[0]->mod_type = "uid";
	mods[0]->mod_values = configs.name;

	mods[1]->mod_op = LDAP_MOD_ADD;
	mods[1]->mod_type = "objectclass";
	mods[1]->mod_values = ldopts.user_object_class;

	mods[2]->mod_op = LDAP_MOD_ADD;
	mods[2]->mod_type = "userPassword";
	mods[2]->mod_values = ldopts.password;

	mods[3]->mod_op = LDAP_MOD_ADD;
	mods[3]->mod_type = "shadowLastChange";
	mods[3]->mod_values = ldopts.shadowlastchange;

	mods[4]->mod_op = LDAP_MOD_ADD;
	mods[4]->mod_type = "shadowMax";
	mods[4]->mod_values = ldopts.shadowmax;

	mods[5]->mod_op = LDAP_MOD_ADD;
	mods[5]->mod_type = "shadowWarning";
	mods[5]->mod_values = ldopts.shadowwarning;

	mods[6]->mod_op = LDAP_MOD_ADD;
	mods[6]->mod_type = "shadowExpire";
	mods[6]->mod_values = ldopts.shadowexpire;

	mods[7]->mod_op = LDAP_MOD_ADD;
	mods[7]->mod_type = "shadowFlag";
	mods[7]->mod_values = ldopts.shadowflag;

	mods[8]->mod_op = LDAP_MOD_ADD;
	mods[8]->mod_type = "loginShell";
	mods[8]->mod_values = ldopts.loginshell;

	mods[9]->mod_op = LDAP_MOD_ADD;
	mods[9]->mod_type = "uidNumber";
	mods[9]->mod_values = ldopts.uidnumber;

	mods[10]->mod_op = LDAP_MOD_ADD;
	mods[10]->mod_type = "gidNumber";
	mods[10]->mod_values = ldopts.gidnumber;

	mods[11]->mod_op = LDAP_MOD_ADD;
	mods[11]->mod_type = "homeDirectory";
	mods[11]->mod_values = ldopts.homedirectory;

	mods[12]->mod_op = LDAP_MOD_ADD;
	mods[12]->mod_type = "gecos";
	mods[12]->mod_values = ldopts.gecos;

	mods[13]->mod_op = LDAP_MOD_ADD;	/* Only so people don't have to turn off */
	mods[13]->mod_type = "homePhone";	/* schema check */
	mods[13]->mod_values = ldopts.homephone;

	mods[14]->mod_op = LDAP_MOD_ADD;
	mods[14]->mod_type = "shadowMin";
	mods[14]->mod_values = ldopts.shadowmin;

	mods[15]->mod_op = LDAP_MOD_ADD;
	mods[15]->mod_type = "shadowInactive";
	mods[15]->mod_values = ldopts.shadowinactive;

	mods[16]->mod_op = LDAP_MOD_ADD;
	mods[16]->mod_type = "cn";
	mods[16]->mod_values = ldopts.cn;

	mods[17]->mod_op = LDAP_MOD_ADD;
	mods[17]->mod_type = "description";
	mods[17]->mod_values = ldopts.description;

	mods[18]->mod_op = LDAP_MOD_ADD;
	mods[18]->mod_type = "mail";
	mods[18]->mod_values = ldopts.mail;

	mods[19]->mod_op = LDAP_MOD_ADD;
	mods[19]->mod_type = "roomnumber";
	mods[19]->mod_values = ldopts.roomnumber;

	mods[20]->mod_op = LDAP_MOD_ADD;
	mods[20]->mod_type = "givenname";
	mods[20]->mod_values = ldopts.givenname;

	mods[21]->mod_op = LDAP_MOD_ADD;
	mods[21]->mod_type = "sn";
	mods[21]->mod_values = ldopts.sn;

	mods[num_mods] = NULL;

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_add_s( %s )\n", ldopts.dn);
	else if (configs.verbose)
		fprintf(stdout, "ldap_add_s( %s )\n", ldopts.dn);

	if ((rc = ldap_add_s(ld, ldopts.dn, mods)) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_add_s\n");
		ldap_perror(ld, " ldap_add_s");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "uid=%s successfully added\n",
		       configs.name[0]);
	else if (configs.verbose)
		fprintf(stdout, "\tuid=%s successfully added\n",
			configs.name[0]);

	return 0;
}

int
do_ldap_user_del()
{
	int rc;

	if ((rc = ldap_delete_s(ld, ldopts.dn)) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_delete_s(ld, %s)\n",
			       ldopts.dn);
		ldap_perror(ld, " ldap_delete_s ");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_delete_s( %s )\n", ldopts.dn);
	else if (configs.verbose)
		fprintf(stdout, "ldap_delete_s( %s )\n", ldopts.dn);

	return 0;
}

int
do_ldap_group_del()
{
	LDAPMessage *result;
	int rc;
	char *attr[2] = { "memberUid", NULL };
  char answer[3];

	if ( ldap_search_s(ld,
                     ldopts.dn,
                     LDAP_SCOPE_SUBTREE,
                     "(memberUid=*)",
                     attr,
                     0,
                     &result) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_search_s\n");
		ldap_perror(ld, "ldap_search_s");
		do_ldap_unbind();
		return 0;
	}

  if ( (ldap_count_entries(ld, result) > 0) && configs.force == 0 ) {
    fprintf(stdout, "%s has existing members, are you sure you want to delete"\
      " the group (y/n) ? ", ldopts.dn);
    fgets(answer, 2, stdin);
    if ( answer[0] != 'y' && answer[0] != 'Y' ) {
      ldap_msgfree(result);
      return 0;
    }
  }

	ldap_msgfree(result);
 
	if ((rc = ldap_delete_s(ld, ldopts.dn)) != LDAP_SUCCESS) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_delete_s( %s )\n", ldopts.dn);
		ldap_perror(ld, " ldap_delete_s ");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_delete_s( %s )\n", ldopts.dn);
	else if (configs.verbose)
		fprintf(stdout, "ldap_delete_s( %s )\n", ldopts.dn);
 
	return 0;
}

int
do_ldap_unbind()
{
	if (ldap_unbind_s(ld) == -1) {
		if (configs.log)
			syslog(LOG_INFO, "Error doing ldap_unbind_s\n");
		perror("ldap_unbind_s");
		return -1;
	}

	if (configs.verbose && configs.log)
		syslog(LOG_INFO, "ldap_unbind(ld)\n");
	else if (configs.verbose)
		fprintf(stdout, "ldap_unbind(ld)\n");

	return 0;
}

char *
get_next_uidnum()
{
	LDAPMessage *result;
	int this_uid_used;
	long num_of_passes;
	long random_number;
	long max_pass;
	char *attr[2] = { "uidnumber", NULL };
	char *new_uid_number;
	char *user_dn;
	double min_uidnumber, max_uidnumber;
	char *temp;

	random_number = 0;
	this_uid_used = 1;
	num_of_passes = 0;

	new_uid_number = (char *) malloc(sizeof (char) * MAX_LEN);
	user_dn = (char *) malloc(sizeof (char) * MAX_LEN);

	if (configs.base)
		strncpy(user_dn, configs.base, MAX_LEN - 1);
	else {
		if (print_config(confs, "user_base") != NULL)
			strncpy(user_dn, print_config(confs, "user_base"),
				MAX_LEN - 1);
		else
			return NULL;
	}

	if ((temp = print_config(confs, "max_uidnumber")) == NULL)
		max_uidnumber = DEFAULT_MAX_UIDNUMBER;
	else if ((max_uidnumber = strtod(temp, NULL)) == 0)
		max_uidnumber = DEFAULT_MAX_UIDNUMBER;

	if ((temp = print_config(confs, "min_uidnumber")) == NULL)
		min_uidnumber = DEFAULT_MIN_UIDNUMBER;
	else if ((min_uidnumber = strtod(temp, NULL)) == 0)
		min_uidnumber = DEFAULT_MIN_UIDNUMBER;

	if (configs.id_max_passes != NULL) {
		if ((max_pass = strtod(configs.id_max_passes, NULL)) == 0)
			max_pass = DEFAULT_ID_MAX_PASSES;
	} else
		max_pass = DEFAULT_ID_MAX_PASSES;

	if (!((max_uidnumber - min_uidnumber > 0) && (min_uidnumber > 0)))
		return NULL;

	srand((unsigned) time(0));

	while (this_uid_used) {
		random_number =
		    min_uidnumber +
		    (long) (rand() / (RAND_MAX + 1.0) *
			    (max_uidnumber - min_uidnumber));
		snprintf(new_uid_number, MAX_LEN - 1, "uidnumber=%ld",
			 random_number);

		if (ldap_search_s
		    (ld, user_dn, LDAP_SCOPE_SUBTREE, new_uid_number, attr, 0,
		     &result) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO, "Error doing ldap_search_s\n");
			ldap_perror(ld, "ldap_search_s");
			do_ldap_unbind();
			return NULL;
		}

		if (++num_of_passes == max_pass)
			return NULL;
		if (!ldap_count_entries(ld, result))
			this_uid_used = 0;
	}

	snprintf(new_uid_number, MAX_LEN - 1, "%ld", random_number);
	free(user_dn);
	free(temp);
	ldap_msgfree(result);
	return (new_uid_number);
}

int
do_ldap_user_cat()
{
	LDAPMessage *result, *e;
	BerElement *ber;
	char *a, *dn;
	char **vals;
	int i, rc, finished, msgid;
	int num_entries = 0;
	struct timeval zerotime;
	char *base_dn;
	char *attr[7] = {
		"uid",
		"uidNumber",
		"gidNumber",
		"gecos",
		"homeDirectory",
		"loginShell",
		NULL
	};
	struct passerd {
		char *uid;
		char *uidNumber;
		char *gidNumber;
		char *gecos;
		char *homeDirectory;
		char *loginShell;
	};
	char *fi = "(uid=*)";

	struct passerd *passentry;

	zerotime.tv_sec = zerotime.tv_usec = 0L;
	base_dn = (char *) malloc(sizeof (char) * MAX_LEN);

	if (configs.base)
		strncpy(base_dn, configs.base, MAX_LEN - 1);
	else {
		if ((base_dn = print_config(confs, "user_base")) == NULL) {
			free(base_dn);
			return 1;
		}
	}

	if ((msgid =
	     ldap_search(ld, base_dn, LDAP_SCOPE_SUBTREE, fi, attr, 0)) == -1) {
		ldap_perror(ld, "ldap_search");
		return (1);
	}

	/* Loop, polling for results until finished */
	finished = 0;

	while (!finished) {
		result = NULL;
		rc = ldap_result(ld, msgid, 0, &zerotime, &result);
		switch (rc) {
		case -1:
			ldap_perror(ld, "ldap_result");
			return (1);
		case 0:
			if (result != NULL) {
				ldap_msgfree(result);
			}
			break;
		default:
			if ((e = ldap_first_entry(ld, result)) == NULL) {
				finished = 1;
				if (result != NULL)
					ldap_msgfree(result);
				continue;
			}

			num_entries++;
			if ((dn = ldap_get_dn(ld, e)) != NULL)
				ldap_memfree(dn);
			passentry = malloc(sizeof (struct passerd));

			for (a = ldap_first_attribute(ld, e, &ber); a != NULL;
			     a = ldap_next_attribute(ld, e, ber)) {
				if ((vals = ldap_get_values(ld, e, a)) != NULL) {
					for (i = 0; vals[i] != NULL; i++) {
						if (strncmp ("uidnumber", btolower(a), 9) == 0) {
							passentry->uidNumber = malloc(sizeof (char) * MAX_LEN);
              snprintf(passentry->uidNumber, MAX_LEN - 1, "%s", vals[i]);
						} else if (strncmp ("uid", btolower(a), 3) == 0) {
							passentry->uid = malloc(sizeof (char) * MAX_LEN);
							snprintf(passentry->uid, MAX_LEN - 1, "%s", vals[i]);
						} else if (strncmp ("gidnumber", btolower(a), 9) == 0) {
							passentry->gidNumber = malloc(sizeof (char) * MAX_LEN);
							snprintf(passentry->gidNumber, MAX_LEN - 1, "%s", vals[i]);
						} else if (strncmp ("gecos", btolower(a), 5) == 0) {
							passentry->gecos = malloc(sizeof (char) * MAX_LEN);
							snprintf(passentry->gecos, MAX_LEN - 1, "%s", vals[i]);
						} else if (strncmp ("homedirectory", btolower(a), 13) == 0) {
							passentry->homeDirectory = malloc(sizeof (char) * MAX_LEN);
							snprintf(passentry->homeDirectory, MAX_LEN - 1, "%s", vals[i]);
						} else if (strncmp ("loginshell", btolower(a), 10) == 0) {
							passentry->loginShell = malloc(sizeof (char) * MAX_LEN);
							snprintf(passentry->loginShell, MAX_LEN - 1, "%s", vals[i]);
						}
					}
					ldap_value_free(vals);
				}
				ldap_memfree(a);
			}
			printf("%s:x:%s:%s:%s:%s:%s\n", passentry->uid,
			       passentry->uidNumber, passentry->gidNumber,
			       passentry->gecos, passentry->homeDirectory,
			       passentry->loginShell);
			free(passentry->uid);
			free(passentry->uidNumber);
			free(passentry->gidNumber);
			free(passentry->gecos);
			free(passentry->homeDirectory);
			free(passentry->loginShell);
			free(passentry);
			ldap_msgfree(result);
		}
		do_other_work();
	}

	free(base_dn);
	return 0;
}

char *
get_next_gidnum()
{
	LDAPMessage *result;
	int this_gid_used;
	long num_of_passes;
	long random_number;
	long max_pass;
	char *attr[2] = { "gidnumber", NULL };
	char *new_gid_number;
	char *user_dn;
	double min_gidnumber, max_gidnumber;
	char *temp;

	random_number = 0;
	this_gid_used = 1;
	num_of_passes = 0;

	new_gid_number = (char *) malloc(sizeof (char) * MAX_LEN);
	user_dn = (char *) malloc(sizeof (char) * MAX_LEN);

	if (configs.base)
		strncpy(user_dn, configs.base, MAX_LEN - 1);
	else {
		if (print_config(confs, "user_base") != NULL)
			strncpy(user_dn, print_config(confs, "user_base"),
				MAX_LEN - 1);
		else
			return NULL;
	}

	if ((temp = print_config(confs, "max_gidnumber")) == NULL)
		max_gidnumber = DEFAULT_MAX_GIDNUMBER;
	else if ((max_gidnumber = strtod(temp, NULL)) == 0)
		max_gidnumber = DEFAULT_MAX_GIDNUMBER;

	if ((temp = print_config(confs, "min_gidnumber")) == NULL)
		min_gidnumber = DEFAULT_MIN_GIDNUMBER;
	else if ((min_gidnumber = strtod(temp, NULL)) == 0)
		min_gidnumber = DEFAULT_MIN_GIDNUMBER;

	if (configs.id_max_passes != NULL) {
		if ((max_pass = strtod(configs.id_max_passes, NULL)) == 0)
			max_pass = DEFAULT_ID_MAX_PASSES;
	} else
		max_pass = DEFAULT_ID_MAX_PASSES;

	if (!((max_gidnumber - min_gidnumber > 0) && (min_gidnumber > 0)))
		return NULL;

	srand((unsigned) time(0));

	while (this_gid_used) {
		random_number =
		    min_gidnumber +
		    (long) (rand() / (RAND_MAX + 1.0) *
			    (max_gidnumber - min_gidnumber));
		snprintf(new_gid_number, MAX_LEN - 1, "gidnumber=%ld",
			 random_number);

		if (ldap_search_s
		    (ld, user_dn, LDAP_SCOPE_SUBTREE, new_gid_number, attr, 0,
		     &result) != LDAP_SUCCESS) {
			if (configs.log)
				syslog(LOG_INFO, "Error doing ldap_search_s\n");
			ldap_perror(ld, "ldap_search_s");
			do_ldap_unbind();
			return NULL;
		}

		if (++num_of_passes == max_pass)
			return NULL;
		if (!ldap_count_entries(ld, result))
			this_gid_used = 0;
	}

	snprintf(new_gid_number, MAX_LEN - 1, "%ld", random_number);
	free(user_dn);
	free(temp);
	ldap_msgfree(result);
	return (new_gid_number);
}

static void
do_other_work()
{
	global_counter++;
}
