/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* main.c
   You have the main, filestuff, and usage functions in this file. Main takes
   care of setting everything up, and calling the rest of the functions to
   add/delete/modify users. Makestuff takes care of making the homedirectory and
   then calling other functions to copy files around, or deleting home
   directory. Usage displays usage
   information. */

/* Written by Blake Matheny */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <bconf.h>
#include <string.h>
#include <ctype.h>
#include <cpu.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <syslog.h>

extern char *conf_file;

#ifdef CONFIGFILE
char *conf_file = CONFIGFILE;
#else
char *conf_file = CONFIG_FILE;
#endif

static char *version = CPU_VERSION;

GLOBAL configs;
LDAPtions ldopts;
CONFIG *confs;
struct bpasswd *passent;

int
main(argc, argv)
int argc;
char *argv[];
{
	extern int optind;
	extern char *optarg;
	char *password = NULL;
	char *hash = NULL;
	int i, c, b, pass, bpass;
	b = c = pass = bpass = 0;
	configs.cipher = configs.verbose = 0;
	configs.file = NULL;

	while ((c =
					bgetopt(argc, argv,
									"b:c:d:D:f:F:g:hH:j:k:l:L:mM:n:p:PrSs:u:vVw:Wy")) != EOF) {
		switch (c) {
		case 'b':									/* different user base then in config ? */
			configs.base = strdup(optarg);
			break;
		case 'c':									/* set gecos comment */
			ldopts.gecos[0] = strdup(optarg);
			break;
		case 'd':									/* set root of home dirs */
			configs.homedir = strdup(optarg);
			break;
		case 'D':									/* set bind dn */
			ldopts.binddn = strdup(optarg);
			break;
		case 'f':									/* set config file */
			configs.file = strdup(optarg);
			break;
		case 'F':									/* first name ? */
			ldopts.givenname[0] = strdup(optarg);
			break;
		case 'g':									/* set gid number */
			ldopts.gidnumber[0] = strdup(optarg);
			break;
		case 'h':									/* display usage */
			usage(argv[0], stdout);
			break;
		case 'H':									/* specify hash at command line */
			configs.hash = strdup(optarg);
			break;
		case 'j':									/* the meberuid to add to group */
			ldopts.memberuid[0] = strdup(optarg);
			ldopts.memberuid[1] = NULL;
			break;
		case 'k':									/* set skel directory */
			configs.skeldir = strdup(optarg);
			break;
		case 'l':									/* change username to this */
			ldopts.newuser[0] = strdup(optarg);
			break;
		case 'L':									/* last name */
			ldopts.sn[0] = strdup(optarg);
			break;
		case 'm':									/* make home directory or not */
			configs.makedir = 1;
			break;
		case 'M':									/* email address */
			ldopts.mail[0] = strdup(optarg);
			break;
		case 'n':									/* new group name */
			ldopts.newgroup[0] = strdup(optarg);
			break;
		case 'p':									/* password supplied */
			password = optarg;
			pass = 1;
			break;
		case 'P':									/* password not supplied */
			pass = 2;
			break;
		case 'r':
			configs.remdir = 1;
			break;
		case 'S':
			pass = 3;
			break;
		case 's':									/* shell to use */
			ldopts.loginshell[0] = strdup(optarg);
			break;
		case 'u':									/* uidnumber */
			ldopts.uidnumber[0] = strdup(optarg);
			break;
		case 'v':									/* enable verbose output */
			++configs.verbose;
			break;
		case 'V':									/* print version */
			fprintf(stdout, "%s\n", version);
			exit(0);
			break;
		case 'w':
			ldopts.bindpass = strdup(optarg);
			break;
		case 'W':
			bpass = 2;
			break;
    case 'y':
      configs.force = 1;
      break;
		default:
			usage(argv[0], stdout);
			break;
		}
	}

	if (optind < argc) {
		char *mopts = NULL;
		if (argv[optind]) {
			mopts = argv[optind];
			while (b < strlen(mopts)) {
				mopts[b] = tolower(mopts[b]);
				++b;
			}
			if (strncmp(mopts, "useradd", 7) == 0) {
				if (pass == 0)
					pass = 2;
				configs.operation = 0;
			} else if (strncmp(mopts, "userdel", 7) == 0) {
				configs.operation = 1;
				ldopts.uidnumber[0] = "";
				ldopts.gidnumber[0] = "";
			} else if (strncmp(mopts, "usermod", 7) == 0) {
				configs.operation = 2;
				if (!ldopts.uidnumber[0])
					ldopts.uidnumber[0] = "";
				if (!ldopts.gidnumber[0])
					ldopts.gidnumber[0] = "";
			} else if (strncmp(mopts, "groupadd", 8) == 0) {
				configs.operation = 3;
				ldopts.uidnumber[0] = "";
			} else if (strncmp(mopts, "groupdel", 8) == 0) {
				configs.operation = 4;
				ldopts.uidnumber[0] = "";
			} else if (strncmp(mopts, "groupmod", 8) == 0) {
				configs.operation = 5;
				ldopts.uidnumber[0] = "";
			} else if (strncmp(mopts, "cat", 3) == 0)
				configs.operation = 6;
			else
				usage(argv[0], stdout);
		} else
			usage(argv[0], stdout);

		if (configs.operation != 6 || argv[optind + 1]) {
			configs.name[0] = argv[optind + 1];
			configs.name[1] = NULL;
		} else if (configs.operation != 6)
			usage(argv[0], stdout);
	} else
		usage(argv[0], stdout);

	if (!configs.file)
		configs.file = conf_file;

	if (bpass == 2)
		ldopts.bindpass = getpass("Enter LDAP Bind Password: ");

	if ((confs = init_config()) == NULL) {
		fprintf(stderr, "Error initializing configuration\n");
		exit(1);
	}

	if (configs.verbose)
		fprintf(stdout, "%s\n", version);

	if (configs.verbose)
		fprintf(stdout, "get_config(%s, \"::\")\n", configs.file);

	if ((i = get_config(confs, configs.file, "::")) == 2) {
		fprintf(stderr,
						"Incorrect format for configuration file," "now exiting.\n");
		exit(1);
	} else if (i == 1) {
		fprintf(stderr,
						"Could not open %s for reading, now exiting.\n", configs.file);
		exit(1);
	} else {
		if ((configs.logfac = print_config(confs, "syslog")) == NULL)
			configs.log = 0;
		else if (strncmp("y", btolower(configs.logfac), 1) == 0)
			configs.log = 1;
		else
			configs.log = 0;

		if (configs.log) {
#ifndef SOLARIS
			openlog("cpu", LOG_PERROR | LOG_ODELAY, LOG_USER);
#else
			openlog("cpu", LOG_ODELAY, LOG_USER);
#endif
		}

		if (do_ldap_init() != 0) {
			if (configs.log) {
				syslog(LOG_INFO, "Error in do_ldap_init()\n");
				closelog();
			} else
				fprintf(stdout, "Error in do_ldap_init()\n");

			exit(1);
		}
	}

	if ((configs.operation == 0 && pass != 3) ||
			(configs.operation == 2 && pass != 0 && pass != 3)) {
		if (!configs.hash) {
			if ((hash = print_config(confs, "hash")) == NULL)
				configs.cipher = 1;
			else if (strncmp("md5", btolower(hash), 3) == 0)	/* hash is md5 */
				configs.cipher = 2;
			else if (strncmp("sha", btolower(hash), 3) == 0)	/* hash is sha1 */
				configs.cipher = 3;
			else if (strncmp("smd5", btolower(hash), 4) == 0)	/* hash is smd5 */
				configs.cipher = 4;
			else if (strncmp("ssha", btolower(hash), 4) == 0)	/* hash is ssha */
				configs.cipher = 5;
			else											/* hash is unsupported, default to crypt */
				configs.cipher = 1;
		} else {
			if (strncmp("crypt", btolower(configs.hash), 5) == 0)
				configs.cipher = 1;			// hash is crypt
			else if (strncmp("md5", btolower(configs.hash), 3) == 0)
				configs.cipher = 2;			// hash is md5
			else if (strncmp("sha", btolower(configs.hash), 3) == 0)
				configs.cipher = 3;			// hash is sha
			else if (strncmp("smd5", btolower(configs.hash), 4) == 0)
				configs.cipher = 4;			// hash is smd5
			else if (strncmp("ssha", btolower(configs.hash), 4) == 0)
				configs.cipher = 5;			// hash is ssha
			else											/* you are crazy, so we're forcing you to use crypt */
				configs.cipher = 1;
		}

		if (pass == 1) {
			ldopts.password[0] = set_password(password);
			ldopts.password[1] = NULL;
		} else if (pass == 2) {
			password = getpass("Enter desired user password: ");
			ldopts.password[0] = set_password(password);
			ldopts.password[1] = NULL;
		} else if (pass != 3) {
			if (configs.log) {
				syslog(LOG_INFO, "Password error, exiting\n");
				closelog();
			} else
				fprintf(stdout, "password error, exiting\n");

			exit(1);
		}
	}

	if (pass == 3) {
		if ((configs.shadfile = print_config(confs, "shadow_file")) == NULL) {
			if (configs.log) {
				syslog(LOG_INFO,
							 "-S requested at command line but"
							 " shadow_file not specified in config file, now" " exiting.\n");
				closelog();
			} else
				fprintf(stdout,
								"-S requested at command line but "
								"shadow_file not specified in config file, " "now exiting.\n");

			exit(1);
		}

		if ((passent = bgetpwent(configs.shadfile, configs.name[0])) == NULL) {
			if (configs.log) {
				syslog(LOG_INFO,
							 "-S specified at command line but %s not "
							 "found in %s, now exiting.\n",
							 configs.name[0], configs.shadfile);
				closelog();
			} else
				fprintf(stdout,
								"-S specified at command line but %s not "
								"found in %s, now exiting.\n",
								configs.name[0], configs.shadfile);

			exit(1);
		} else {
			ldopts.password[0] = (char *) malloc(sizeof (LDAPtions));
			snprintf(ldopts.password[0], MAX_LEN, "{crypt}%s", passent->pw_passwd);
		}

		if (configs.log && configs.verbose)
			syslog(LOG_INFO, "%s\n", ldopts.password[0]);
		else if (configs.verbose)
			fprintf(stdout, "%s\n", ldopts.password[0]);

		ldopts.password[1] = NULL;
	}

	if (configs.operation == 0)
		do_ldap_user_add();
	else if (configs.operation == 1)
		do_ldap_user_del();
	else if (configs.operation == 2)
		do_ldap_user_mod();
	else if (configs.operation == 3) {
		if (!ldopts.memberuid[0])
			do_ldap_group_add();
		else
			do_ldap_member_mod(1);
	} else if (configs.operation == 4) {
		if (!ldopts.memberuid[0])
			do_ldap_group_del();
		else
			do_ldap_member_mod(0);
	} else if (configs.operation == 5)
		do_ldap_group_mod();
	else if (configs.operation == 6)
		do_ldap_user_cat();
	else
		usage(argv[0], stdout);

	do_ldap_unbind();

	if (configs.makedir == 1 && configs.operation == 0)
		filestuff(0);

	if (configs.remdir == 1 && configs.operation == 1)
		filestuff(1);

	if (configs.log)
		closelog();

	destroy_config(confs);

	exit(0);
}

int
filestuff(filefunct)
int filefunct;
{
	char *homedirectory;
	char *buf;

	if (!configs.homedir) {
		if ((configs.homedir = print_config(confs, "home_directory")) == NULL) {
			if (configs.log)
				syslog(LOG_INFO,
							 "home_directory not specified at command "
							 "line and not found in config file. Can not make home "
							 "directory.\n");
			else
				fprintf(stdout,
								"home_directory not specified at command "
								"line and not found in config file. Can not "
								"make home directory.\n");
			return 1;
		}
	}

	homedirectory = (char *) malloc(sizeof (char) * MAX_LEN);
	sprintf(homedirectory, "%s/%s", configs.homedir, configs.name[0]);

	if (filefunct == 0) {
		if (configs.verbose && configs.log)
			syslog(LOG_INFO, "Now making %s\n", homedirectory);
		else if (configs.verbose)
			fprintf(stdout, "Now making %s\n", homedirectory);

		if ((configs.def_dir_perm = print_config(confs, "def_dir_perm")) == NULL) {
			if (mkdir(homedirectory, atoo(configs.def_dir_perm)) == -1) {
				if (configs.log)
					syslog(LOG_INFO, "Error doing mkdir(%s)\n", homedirectory);
				perror("mkdir: ");
				return 1;
			}
		} else {
			configs.def_dir_perm = "0700";
			if (mkdir(homedirectory, atoo(configs.def_dir_perm)) == -1) {
				if (configs.log)
					syslog(LOG_INFO, "Error doing mkdir(%s)\n", homedirectory);
				perror("mkdir: ");
				return 1;
			}
		}

		if (chown
				(homedirectory, atoi(ldopts.uidnumber[0]),
				 atoi(ldopts.gidnumber[0])) == -1) {
			if (configs.log)
				syslog(LOG_INFO, "Error doing chown(%s)\n", homedirectory);
			perror("chown: ");
		}

		if (!configs.skeldir) {
			if ((configs.skeldir = print_config(confs, "skel_dir")) == NULL) {
				if (!configs.log)
					fprintf(stdout,
									"No skel_dir specified in config file or at "
									"command line, not copying any files.\n");
				else
					syslog(LOG_INFO,
								 "No skel_dir specified in config file or at"
								 "command line, not copying any files.\n");
			}
		}
		copy(configs.skeldir, homedirectory);
	}

	if (filefunct == 1) {
		fprintf(stdout, "Are you sure you want to delete %s (y/n) ? ",
						homedirectory);
		buf = (char *) malloc(sizeof (char *) * 5);
		scanf("%s", buf);

		if (strncmp(&buf[0], "y", 1) == 0) {
			if (!configs.log)
				printf("Removing %s\n", homedirectory);
			else
				syslog(LOG_INFO, "Removing %s\n", homedirectory);
			remdir(homedirectory);
		} else
			printf("Good choice :)\n");
	}

	return 0;
}

void
usage(progName, OUTFILE)
char *progName;
FILE *OUTFILE;
{
	fprintf(OUTFILE, "  Usage - %s user{add|mod|del} name [options]\n", progName);
	fprintf(OUTFILE, "  Usage - %s group{add|mod|del} name [options]\n",
					progName);
	fprintf(OUTFILE, "  Usage - %s cat\n", progName);
	fprintf(OUTFILE, "\tOPTIONS\n");
	fprintf(OUTFILE, "\t-b\tBase to add user to\n");
	fprintf(OUTFILE, "\t-c\tGecos Comment\n");
	fprintf(OUTFILE, "\t-d\tUsers home directory, see man page.\n");
	fprintf(OUTFILE, "\t-D\tBind DN\n");
	fprintf(OUTFILE, "\t-f\tconfig file\n");
	fprintf(OUTFILE, "\t-F\tFirst Name\n");
	fprintf(OUTFILE, "\t-g\tgid number\n");
	fprintf(OUTFILE, "\t-h\tThis help\n");
	fprintf(OUTFILE, "\t-H\tHash to use (crypt|md5|sha|smd5|ssha)\n");
	fprintf(OUTFILE, "\t-j\tUser to add to or delete from existing group\n");
	fprintf(OUTFILE, "\t-k\tSkeleton Directory\n");
	fprintf(OUTFILE, "\t-l\tNew username (for usermod only)\n");
	fprintf(OUTFILE, "\t-L\tLast Name\n");
	fprintf(OUTFILE, "\t-m\tMake users home directory\n");
	fprintf(OUTFILE, "\t-M\tEmail Address\n");
	fprintf(OUTFILE, "\t-n\tNew groupname (for groupmod only)\n");
	fprintf(OUTFILE, "\t-p\tUser Password - requires argument\n");
	fprintf(OUTFILE, "\t-P\tUser Password - prompts user for password\n");
	fprintf(OUTFILE, "\t-r\tRemove users home directory (y/n)\n");
	fprintf(OUTFILE, "\t-s\tShell\n");
	fprintf(OUTFILE, "\t-S\tShadow Password - use from file specified by\n"
					"\t\tshadow_file in cpu.cfg\n");
	fprintf(OUTFILE, "\t-u\tUID Number\n");
	fprintf(OUTFILE, "\t-v\tVerbose\n");
	fprintf(OUTFILE, "\t-V\tVersion\n");
	fprintf(OUTFILE, "\t-w\tBind Password - requires argument\n");
	fprintf(OUTFILE, "\t-W\tBind Password - prompts user for password\n");
  fprintf(OUTFILE, "\t-y\tAssume yes to questions\n");
	exit(0);
}

char *
getToken(str, delims)
char **str;
const char *delims;
{
	char *token;

	if (*str == NULL) {
		/* No more tokens */
		return NULL;
	}

	token = *str;
	while (**str != '\0') {
		if (strchr(delims, **str) != NULL) {
			**str = '\0';
			(*str)++;
			return token;
		}
		(*str)++;
	}
	/* There is no other token */
	*str = NULL;
	return token;
}

char *
btolower(baz)
char *baz;
{
	int length = strlen(baz);
	int i = 0;
	char *temp;

	temp = (char *) malloc(sizeof (char) * length);

	for (i = 0; i < length; i++)
		temp[i] = tolower(baz[i]);

	return temp;

}
