/* CPU Change Password Utility
   Copyright (C) 2001 dbaseiv.net

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA */

/* password.c
   Function in here take care of setting up a salt, and then hashing a password
   with either crypt, md5, sha, ssha or smd5. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <bconf.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <syslog.h>
#ifndef FREEBSD
  #include <crypt.h>
#endif
#include <cpu.h>
#include <md5.h>

char salt[] = "$1$........";
char csalt[] = "........";

char *
get_salt()
{
	unsigned long seed[2];
	const char *const seedchars =
	    "./0123456789ABCDEFGHIJKLMNOPQRST"
	    "UVWXYZabcdefghijklmnopqrstuvwxyz";
	int i;

	seed[0] = time(NULL);
	seed[1] = getpid() ^ (seed[0] >> 14 & 0x30000);

	for (i = 0; i < 8; i++)
		salt[3 + i] = seedchars[(seed[i / 5] >> (i % 5) * 6) & 0x3f];
	return salt;
}

char *
cget_salt()
{
	unsigned long seed[2];
	const char *const seedchars =
	    "./0123456789ABCDEFGHIJKLMNOPQRST"
	    "UVWXYZabcdefghijklmnopqrstuvwxyz";
	int i;

	seed[0] = time(NULL);
	seed[1] = getpid() ^ (seed[0] >> 14 & 0x30000);

	for (i = 0; i < 8; i++)
		csalt[i] = seedchars[(seed[i / 5] >> (i % 5) * 6) & 0x3f];
	return csalt;
}

char *
set_password(passphrase)
char *passphrase;
{
	char *password;

	if (configs.cipher == 1) {
		password = crypt(passphrase, cget_salt());
        
		if (configs.verbose && configs.log)
            syslog(LOG_INFO, "{crypt}%s\n", password);
        else if (configs.verbose)
			fprintf(stdout, "{crypt}%s\n", password);

		return password;
	} else if (configs.cipher == 2) {
		password = md5_hash(passphrase);

		if (configs.verbose && configs.log)
            syslog(LOG_INFO, "{md5}%s\n", password);
        else if ( configs.verbose )
			fprintf(stdout, "{md5}%s\n", password);

		return password;
	} else if (configs.cipher == 3) {
		password = sha1_hash(passphrase);

		if (configs.verbose && configs.log)
            syslog(LOG_INFO, "{sha}%s\n", password);
        else if ( configs.verbose )
			fprintf(stdout, "{sha}%s\n", password);

		return password;
    } else if (configs.cipher == 4) {
        password = smd5_hash(passphrase, get_salt());

        if (configs.verbose && configs.log)
            syslog(LOG_INFO, "{smd5}%s\n", password);
        else if ( configs.verbose )
            fprintf(stdout, "{smd5}%s\n", password);

        return password;
    } else if (configs.cipher == 5) {
        password = ssha1_hash(passphrase, get_salt());

        if (configs.verbose && configs.log)
            syslog(LOG_INFO, "{ssha}%s\n", password);
        else if ( configs.verbose )
            fprintf(stdout, "{ssha}%s\n", password);

        return password;
	} else {
        if ( configs.log && configs.verbose )
		    syslog(LOG_INFO, "unknown hash, resorting to crypt\n");
        else
            fprintf(stdout, "unknown hash, resorting to crypt\n");
		password = crypt(passphrase, cget_salt());
        return password;
	}
	return NULL;
}
